/*
 * application.java
 *
 * Created on 03 January 2004, 15:49
 */

package jim.common;

import java.io.FileInputStream;
import java.sql.Connection;
import java.util.Hashtable;
import java.util.Properties;
import java.util.ResourceBundle;

/**
 * Basis of all applications.
 * @author briggsj
 * @deprecated Use {@link BaseApplication} instead
 */
public abstract class JimApplication {
    
    private Hashtable<String,Object> hash;
    
    /** Holds value of property props. */
    private Properties props;
    
    /**
     * Holds value of property dbname.
     */
    private String dbname;
    
    /** Creates a new instance of application */
    public JimApplication() {
    }
    
    final synchronized void init() throws Exception {
        init0();
    }
    
    public void init0() {
        Trace.print("Application initialisation");
    }
    
    public void setProperties(String appPropFile) {
        Properties sysProps = System.getProperties();
        props = new Properties(sysProps);
        try {
            props.load(new FileInputStream(appPropFile));
        } catch (Exception e) {
            //ignore
        }
        System.setProperties(props);
    }
    
    public void setProperties(String[] appPropFiles) {
        Properties props = System.getProperties();
        try {
            for (int i=0; i<appPropFiles.length; i++) {
                props = new Properties(props);
                props.load(new FileInputStream(appPropFiles[i]));
            }
        } catch (Exception e) {
            //ignore
        }
        System.setProperties(props);
    }
    
    //    public synchronized void initApplication() throws Exception {
    //    }
    //
    //    public synchronized void initSession() throws Exception {
    //    }
    //
    //    public synchronized void initRequest() throws Exception {
    //    }
    
    public boolean isDeveloper() {
        return true;
    }
    
    public void setAttribute(String name, Object value) {
        hash.put(name, value);
    }
    
    public Object getAttribute(String name) {
        return hash.get(name);
    }
    
    public void removeAttribute(String name) {
        hash.remove(name);
    }
    
    /** Getter for property props.
     * @return Value of property props.
     *
     */
    public Properties getProps() {
        return this.props;
    }
    
    /** Setter for property props.
     * @param props New value of property props.
     *
     */
    public void setProps(Properties props) {
        this.props = props;
    }
    
    /** Getter for property name.
     * @return Value of property name.
     *
     */
    public abstract String getName();
    

    
    /** Write out a log file entry */
    public void doLog() {
        //open log file
        //write all available details to it
    }
    
    /**
     * Getter for property dbname.
     * @return Value of property dbname.
     */
    public String getDbname() {
        return this.dbname;
    }
    
    /**
     * Setter for property dbname.
     * @param dbname New value of property dbname.
     */
    public void setDbname(String dbname) {
        this.dbname = dbname;
    }
    
    protected abstract ResourceBundle getResources();
    
    /**
     * 
     * @throws jim.common.JimFatalException 
     * @return 
     * @deprecated Trying to get rid of the whole JimApplication class. Use {@link BaseApplication} instead
     */
    public abstract Connection getConn() throws JimFatalException;
    
}
