/*
 * BundleMessageFetcher.java
 *
 * Created on 19 April 2005, 12:04
 */

package jim.common.mail;

import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Fetch email messages from a resource bundle
 * @author briggsj
 */
public class BundleMessageFetcher extends GenericMessageFetcher {
    
    private ResourceBundle bundle;
    private String bundleName;
    
    /**
     * Creates a new instance of BundleMessageFetcher linked to the specified resource bundle
     * @param bundleName Name of the resource bundle to use
     */
    public BundleMessageFetcher(String bundleName) {
        bundle = ResourceBundle.getBundle(bundleName);
        this.bundleName = bundleName;
    }
    
    /**
     * Fetch a message component
     * @param messageName Name of message to fetch
     * @throws MessageException Message (or component of message) not found
     * @return {@link EmailMessage}
     */
    public String fetchString(String messageName) throws MessageException {
        String text;
        if (!"".equals(messageName)) {
            try {
                text = bundle.getString(messageName);
            } catch (MissingResourceException e) {
                throw new MessageException("Cannot find message " + messageName + " in resource bundle " + bundleName);
            }
        } else {
            throw new MessageException("No message name specified in resource bundle " + bundleName);
        }
        return text;
    }
    
    /**
     * Load all the components that are contained in the message pool
     * @throws MessageException If there are database problems
     */    
    public void fetchAllStrings() throws MessageException {
        Enumeration en = bundle.getKeys();
        while (en.hasMoreElements()) {
            String key = (String)en.nextElement();
            load(key, false, null);
        }
    }
    
    public void storeString(String stringName, String value) {
        throw new MessageException("Cannot update resource bundle");
    }
    
}
