/*
 * DBMessageFetcher.java
 *
 * Created on 19 April 2005, 12:04
 */

package jim.common.mail;

import java.sql.ResultSet;
import java.sql.SQLException;

import jim.util.SqlUtils;

/**
 * Fetch email messages from a database
 * @author briggsj
 */
public class DBMessageFetcher extends GenericMessageFetcher {
    
    private SqlUtils sql;
    private String selectOneRow;
    private String selectAllRows;
    
    /**
     * Creates a new instance of DBMessageFetcher linked to the specified database connection
     * @param conn Database connection to use
     * @param tableName Name of database table containing message components
     * @param keyName Name of column in table <CODE>tableName</CODE> containing component key
     * @param valueName Name of column in table <CODE>tableName</CODE> containing component value
     */
    public DBMessageFetcher(SqlUtils sql, String tableName, String keyName, String valueName) {
        init(sql, tableName, keyName, valueName);
    }
    
    /**
     * Initialisation
     */
    private void init(SqlUtils sql, String tableName, String keyName, String valueName) {
        this.selectOneRow = "SELECT " + keyName + ", " + valueName + " FROM " + tableName + " WHERE lower(" + keyName + ") = lower(?)";
        this.selectAllRows = "SELECT " + keyName + ", " + valueName + " FROM " + tableName;
        this.sql = sql;
    }
    
    /**
     * Fetch a message component
     * @param messageName Name of message to fetch
     * @throws MessageException Message (or component of message) not found
     * @return {@link EmailMessage}
     */
    public String fetchString(String messageName) throws MessageException {
        String text;
        if (!"".equals(messageName)) {
            try {
                ResultSet rs = sql.selectRS(selectOneRow, new Object[]{messageName});
                if (rs.next()) {
                    text = rs.getString(2);
                    if (text == null) {
                        text = "";
                    }
                    return text;
                } else {
                    throw new MessageException("Cannot find message " + messageName + " in database ");
                }
            } catch (SQLException e) {
                throw new MessageException("Cannot find message " + messageName + " in database ");
            }
        } else {
            throw new MessageException("No message name specified in database ");
        }
    }
    
    
    /**
     * Close connection.
     * @throws Throwable Anything can happen!
     */
    protected void finalize() throws Throwable {
        super.finalize();
    }
    
    /**
     * Load all strings from the database
     * @throws MessageException if there are database problems
     */
    public void fetchAllStrings() throws MessageException {
        try {
            ResultSet rs = sql.selectRS(selectAllRows);
            while (rs.next()) {
                String key = rs.getString(1);
                String value = rs.getString(2);
                this.setString(key, value);
            }
            rs.close();
        } catch (SQLException e) {
            throw new MessageException("Problems reading components from database", e);
        }
    }
    
    /**
     * Store a string in the database
     * @param messageName the name of the string
     * @param value its value
     */
    public void storeString(String messageName, String value) throws MessageException {
        if (messageName != null && !"".equals(messageName)) {
            try {
                ResultSet rs = sql.selectRSmod(selectOneRow, new Object[]{messageName});
                if (rs.next()) {
                    //found = update
                    rs.updateString(2, value);
                    rs.updateRow();
                } else {
                    //not found = insert
                    rs.moveToInsertRow();
                    rs.updateString(1, messageName);
                    rs.updateString(2, value);
                    rs.insertRow();
                }
            } catch (SQLException e) {
                throw new MessageException("Cannot find message " + messageName + " in database ", e);
            }
        } else {
            throw new MessageException("No message name specified in database ");
        }
    }
    
}
