/*
 * PumsBaseAction.java
 *
 * Created on 17 August 2004, 16:17
 */

package jim.util.struts;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import jim.common.AppInfo;
import jim.common.BaseApplication;
import jim.common.JimConstants;
import jim.common.JimFatalException;
import jim.common.Trace;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessages;


/**
 * Things to be done in all actions in all JimApp applications
 * @author briggsj
 */
public abstract class JimBaseAction extends Action {
    
    /**
     * Creates a new instance of JimBaseAction
     */
    public JimBaseAction() {
        super();
    }
    
    /**
     * Check to see whether this action can be performed by the user
     * @param su All the useful info
     * @return whether or not the user is allowed to perform this action
     */
    protected boolean isRoleOK(StrutsUtils su) {
        ActionMapping mapping = su.getMapping();
        HttpServletRequest request = su.getRequest();
        
        String roles[] = mapping.getRoleNames();
        if ((roles == null) || (roles.length < 1)) {
            Trace.print("No roles specified for " + mapping.getPath());
            return true; // OK, if no roles specified (default)
        }
        
        for (int i = 0; i < roles.length; i++) {
            if (request.isUserInRole(roles[i])) {
                Trace.print("User " + request.getRemoteUser() + " has role " + roles[i]);
                return true;
            }
        }
        Trace.print("User " + request.getRemoteUser() + " has no required role: " + roles.toString());
        return false;
    }
    
    
    /**
     * The base action for all actions in all jim apps
     * @param mapping One of the standard Struts action values
     * @param form One of the standard Struts action values
     * @param request One of the standard Struts action values
     * @param response One of the standard Struts action values
     * @return Struts Action forward
     * @throws java.io.IOException standard if any IO problem occurs
     * @throws javax.servlet.ServletException standard if any servlet problem occurs
     */
    public synchronized final ActionForward execute(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws IOException, ServletException {
        Trace.setLevel(0);
        Trace.print(mapping.getPath());
        
        StrutsUtils su = new StrutsUtils();
        BaseApplication app = getApplication(request);
        su.setApp(app);
        
        JimActionErrors errors = new JimActionErrors();
        JimActionErrors messages = new JimActionErrors();
        su.setErrors(errors);
        su.setMessages(messages);
        su.setMapping(mapping);
        su.setForm(form);
        su.setRequest(request);
        su.setResponse(response);
        su.setServletContext(this.getServlet().getServletContext());
        fixAppInfo(su);
        
        ActionForward af;
        try {
            if (isRoleOK(su)) {
                af = doExecute(su);
                if (af == null) {
                    ActionMapping m = su.getMapping();
                    throw new JimFatalException("No ActionForward returned by " + m.getType() + " (" + m.getPath() + ")");
                }
            } else {
                errors.addGlobal("errors.notinrole", request.getRemoteUser());
                af = mapping.findForward("notinrole");
            }
        } catch (JimFatalException e) {
            errors.addException(e);
            Throwable t = e.getCause();
            if (t == null) {
                t = e;
            } else {
                errors.addException(t);
            }
            af = mapping.findForward("fatal");
        }
        saveErrors(request, (ActionMessages)errors);
        saveMessages(request, messages);
        
        return af;
    }
    
    private void fixAppInfo(final StrutsUtils su) {
        BaseApplication app = su.getApp();
        AppInfo appInfo = (AppInfo)su.getAttribute(JimConstants.APPINFO, StrutsUtils.SESSION_SCOPE);
        if (appInfo == null) {
            appInfo = app.getAppInfo();
            if (appInfo == null) {
                appInfo = new AppInfo();
                app.setAppInfo(appInfo);
            }
            su.setAttribute(JimConstants.APPINFO, appInfo, StrutsUtils.SESSION_SCOPE);
            appInfo.put("app.version", app.getVersion());
        }
    }
    
    /**
     * Get the application-specific root of the MVC model
     * @param request needed to access the application context
     * @return the application object
     */
    protected BaseApplication getApplication(HttpServletRequest request) {
        BaseApplication app;
        HttpSession session = request.getSession();
        
        app = (BaseApplication)session.getAttribute(JimConstants.APPLICATION);
        if (app == null) {
            app = createApplication();
            session.setAttribute(JimConstants.APPLICATION, app);
        }
        return app;
    }
    
    /**
     * Create a new application-specific application object
     * @return the application object
     */
    abstract protected BaseApplication createApplication();
    
    /**
     * This needs to be implemented by all actions
     * @param su Bundle of standard Struts objects
     * @throws java.io.IOException if any IO problems
     * @throws javax.servlet.ServletException if any servlet problems
     * @throws jim.common.JimFatalException if any application specific problem
     * @return an ActionForward
     */
    abstract public ActionForward doExecute(StrutsUtils su) throws IOException, ServletException, JimFatalException;
    
}